import skycoin

MaxUint64 = 0xFFFFFFFFFFFFFFFF
Million = 1000000
MaxUint16 = 0xFFFF


def makecipher_PubKeyAndcipher_SecKey():
    p = skycoin.cipher_PubKey()
    s = skycoin.cipher_SecKey()
    err = skycoin.SKY_cipher_GenerateKeyPair(p, s)
    return p, s


def makeAddress():
    p, s = makecipher_PubKeyAndcipher_SecKey()
    a = skycoin.cipher__Address()
    assert skycoin.SKY_cipher_AddressFromPubKey(p, a) == skycoin.SKY_OK
    return a


def makeTransactionFromUxOut(ux, s):
    _, handle = skycoin.SKY_coin_Create_Transaction()
    _, tx = skycoin.SKY_coin_GetTransactionObject(handle)
    h = skycoin.cipher_SHA256()
    assert skycoin.SKY_cipher_SecKey_Verify(s) == skycoin.SKY_OK
    assert skycoin.SKY_coin_UxOut_Hash(ux, h) == skycoin.SKY_OK
    r = skycoin.SKY_coin_Transaction_PushInput(handle, h)
    assert skycoin.SKY_coin_Transaction_PushOutput(
        handle, makeAddress(), int(1e6), int(50)) == skycoin.SKY_OK
    assert skycoin.SKY_coin_Transaction_PushOutput(
        handle, makeAddress(), int(5e6), int(50)) == skycoin.SKY_OK
    secKeys = []
    secKeys.append(s)
    assert skycoin.SKY_coin_Transaction_SignInputs(
        handle, secKeys) == skycoin.SKY_OK
    assert skycoin.SKY_coin_Transaction_UpdateHeader(handle) == skycoin.SKY_OK
    return handle, tx


def makeUxBodyWithSecret():
    p = skycoin.cipher_PubKey()
    s = skycoin.cipher_SecKey()
    assert skycoin.SKY_cipher_GenerateKeyPair(p, s) == skycoin.SKY_OK
    uxb = skycoin.coin__UxBody()
    err, b = skycoin.SKY_cipher_RandByte(128)
    assert err == skycoin.SKY_OK
    h = skycoin.cipher_SHA256()
    assert skycoin.SKY_cipher_SumSHA256(b, h) == skycoin.SKY_OK
    assert h.assignTo(uxb.SrcTransaction) == None
    a = skycoin.cipher__Address()
    assert skycoin.SKY_cipher_AddressFromPubKey(p, a) == skycoin.SKY_OK
    uxb.Address = a
    uxb.Coins = int(1e6)
    uxb.Hours = int(100)
    return uxb, s


def makeUxOutWithSecret():
    body, sec = makeUxBodyWithSecret()
    uxo = skycoin.coin__UxOut()
    uxh = skycoin.coin__UxHead()
    uxh.Time = 100
    uxh.BkSeq = 2
    uxo.Head = uxh
    uxo.Body = body
    return uxo, sec


def makeTransaction():
    ux, s = makeUxOutWithSecret()
    return makeTransactionFromUxOut(ux, s)


def makeTransactions(n):
    _, handle = skycoin.SKY_coin_Create_Transactions()
    for i in range(n):
        thandle, _ = makeTransaction()
        assert skycoin.SKY_coin_Transactions_Add(
            handle, thandle) == skycoin.SKY_OK
    err, count = skycoin.SKY_coin_Transactions_Length(handle)
    assert err == skycoin.SKY_OK
    assert count == n
    return handle


def copyTransaction(handle):
    handle2 = skycoin.coin__Transaction()
    err, handle2 = skycoin.SKY_coin_Transaction_Copy(
        handle)
    assert err == skycoin.SKY_OK
    assert skycoin.SKY_coin_Transaction_Verify(handle2) == skycoin.SKY_OK
    err, ptx = skycoin.SKY_coin_GetTransactionObject(handle2)
    assert err == skycoin.SKY_OK
    return handle2, ptx


def makeEmptyTransaction():
    _, handle = skycoin.SKY_coin_Create_Transaction()
    return handle


def makeUxOut():
    pOut, _ = makeUxOutWithSecret()
    return pOut


def equalTransactions(handle1, handle2):
    err, size1 = skycoin.SKY_coin_Transactions_Length(handle1)
    assert err == skycoin.SKY_OK
    err, size2 = skycoin.SKY_coin_Transactions_Length(handle2)
    assert err == skycoin.SKY_OK
    if size1 != size2:
        return 1

    for i in range(int(size1 - 1)):
        err, tx1 = skycoin.SKY_coin_Transactions_GetAt(handle1, i)
        assert err == skycoin.SKY_OK
        err, tx2 = skycoin.SKY_coin_Transactions_GetAt(handle2, i)
        assert err == skycoin.SKY_OK
        err, tx1_obj = skycoin.SKY_coin_GetTransactionObject(tx1)
        assert err == skycoin.SKY_OK
        err, tx2_obj = skycoin.SKY_coin_GetTransactionObject(tx2)
        assert err == skycoin.SKY_OK
        assert tx1_obj == tx2_obj
        i += 1

    return 0


def badFeeCalculator(transaction):
    return skycoin.SKY_ERROR, 0


def calc(transaction):
    return 0, 1


def feeCalc(transaction):
    return skycoin.SKY_OK, 0


def overflowCalc(transaction):
    return 0, MaxUint64
    return 1, 0


def fix121FeeCalculator(transaction):
    return skycoin.SKY_OK, 121


def makeUxBody():
    return makeUxBodyWithSecret()[0]


def make_UxOut():
    return makeUxOutWithSecret()[0]


def makeUxArray(n):
    lista = []
    for _ in range(n):
        lista.append(make_UxOut())
    return lista


def err_CoinHours_Overflow(p0):
    if MaxUint16 < p0:
        return 67108864


def RandSHA256():
    err, sha = skycoin.SKY_cipher_RandByte(128)
    assert err == skycoin.SKY_OK
    sh = skycoin.cipher_SHA256()
    err = skycoin.SKY_cipher_SumSHA256(sha, sh)
    assert err == skycoin.SKY_OK
    return sh


def makeKeysAndAddress():
    ppubkey = skycoin.cipher_PubKey()
    pseckey = skycoin.cipher_SecKey()
    err = skycoin.SKY_cipher_GenerateKeyPair(ppubkey, pseckey)
    assert err == skycoin.SKY_OK
    paddress = skycoin.cipher__Address()
    err = skycoin.SKY_cipher_AddressFromPubKey(ppubkey, paddress)
    assert err == skycoin.SKY_OK
    return err, ppubkey, pseckey, paddress



import skycoin
import tests.utils
import ctypes


class StrTmp():
    a = 0
    b = ctypes.c_uint32(0)
    err = 0

    def __init__(self, a, b, err=0):
        self.a = a
        self.b = b
        self.err = err


def test_Test64BitIntToUint32():
    cases = []
    cases.append(StrTmp((1 << 32), 0, skycoin.SKY_ErrIntOverflowsUint32))
    cases.append(StrTmp((1 << 32 - 2), (1 << 32 - 2)))
    for tc in cases:
        err, x = skycoin.SKY_util_IntToUint32(tc.a)
        if err != skycoin.SKY_OK:
            assert tc.err == err
        else:
            assert ctypes.c_uint32(tc.b).value == ctypes.c_uint32(x).value



"""
 Skycoin python extension

"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages, Extension
from setuptools.command.build_ext import build_ext
from distutils.errors import DistutilsSetupError

# To use a consistent encoding
from codecs import open
from os import path
import os
import subprocess
import platform
import sys
from skycoin import __version__

script_dirname = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(script_dirname, "README.md"), encoding="utf-8") as f:
    long_description = f.read()


class skycoin_build_ext(build_ext, object):
    def build_extension(self, ext):

        if ext.name != "_skycoin":
            # Handle unspecial extensions with the parent class' method
            super(skycoin_build_ext, self).build_extension(ext)
        else:
            # Handle special extension
            sources = ext.sources
            if sources is None or not isinstance(sources, (list, tuple)):
                raise DistutilsSetupError(
                    "in 'ext_modules' option (extension '%s'), "
                    "'sources' must be present and must be "
                    "a list of source filenames" % ext.name
                )
            sources = list(sources)

            if len(sources) > 1:
                sources_path = os.path.commonprefix(sources)
            else:
                sources_path = os.path.dirname(sources[0])
            sources_path = os.path.realpath(sources_path)
            if not sources_path.endswith(os.path.sep):
                sources_path += os.path.sep

            if not os.path.exists(sources_path) or not os.path.isdir(sources_path):
                raise DistutilsSetupError(
                    "in 'extensions' option (extension '%s'), "
                    "the supplied 'sources' base dir "
                    "must exist" % ext.name
                )

            make_path = os.path.realpath(os.path.join(sources_path, ".."))

            make_process = subprocess.Popen(
                "make build-libc-swig",
                cwd=make_path,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                shell=True,
            )
            stdout, stderr = make_process.communicate()
            print("stdout:")
            sys.stderr.write(str(stdout))
            if len(stderr) > 0:
                print("stderr:")
                sys.stderr.write(str(stderr))
            # After making the library build the c library's
            # python interface with the parent build_extension method
            super(skycoin_build_ext, self).build_extension(ext)


skypath = path.join(*("gopath/src/github.com/skycoin/libskycoin".split("/")))
lib_path = path.join(skypath, "build", "libskycoin")
library_file = path.join(lib_path, "libskycoin.a")
extra_link_args = []
if platform.system() == "Darwin":
    extra_link_args += ["-framework", "Foundation", "-framework", "Security"]
extra_link_args.append(library_file)

setup(
    name='pyskycoin',  # Required
    version=__version__,  # Required
    description="Skycoin Python Library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/skycoin/pyskycoin",
    author="Ratmil Torres",  # Optional
    author_email="skycoin@simelo.tech",
    setup_requires=["pytest-runner"],
    tests_require=["pytest", "urllib3", "certifi"],
    classifiers=[
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    keywords="skycoin crypto coin currency blockchain",  # Optional
    py_modules=["skycoin"],
    packages=find_packages(exclude=["contrib", "docs", "tests"]),  # Required
    install_requires=[],
    extras_require={"dev": ["check-manifest"], "test": ["coverage"]},  # Optional
    package_data={},
    entry_points={"console_scripts": []},
    cmdclass={"build_ext": skycoin_build_ext},
    ext_modules=[
        Extension(
            "_skycoin",
            ["swig/pyskycoin_wrap.c"],
            include_dirs=["swig/include", path.join(skypath, "include")],
            extra_link_args=extra_link_args,
            depends=[],
        )
    ],
)
